"""Aligo class"""
import json
import logging
from typing import Callable, Dict

from .Album import Album
from .Audio import Audio
from .Compress import Compress
from .Copy import Copy
from .Copy import aligo_config_folder
from .Create import Create
from .CustomShare import CustomShare
from .Download import Download
from .Drive import Drive
from .Duplicate import Duplicate
from .File import File
from .Move import Move
from .Other import Other
from .Recyclebin import Recyclebin
from .Search import Search
from .Share import Share
from .Star import Star
from .SyncFolder import SyncFolder
from .Update import Update
from .Video import Video
from aligo.types import EMailConfig


class Aligo(
    Audio,
    Album,
    Compress,
    Copy,
    Create,
    CustomShare,
    Download,
    Drive,
    Duplicate,
    File,
    Move,
    Other,
    Recyclebin,
    Search,
    Share,
    Star,
    SyncFolder,
    Update,
    Video,
):
    """阿里云盘"""

    def __init__(
            self,
            name: str = 'aligo', *,
            refresh_token: str = None,
            show: Callable[[str], None] = None,
            level: int = logging.DEBUG,
            use_aria2: bool = False,
            proxies: Dict = None,
            port: int = None,
            email: EMailConfig = None,
            request_failed_delay: float = 3,
            requests_timeout: float = None,
            login_timeout: float = None,
            re_login: bool = True
    ):
        """
        Aligo
        :param name: (可选, 默认: aligo) 配置文件名称, 便于使用不同配置文件进行身份验证
        :param refresh_token:
        :param show: (可选) 显示二维码的函数
        :param level: (可选) 控制控制台输出
        :param use_aria2: [bool] 是否使用 aria2 下载
        :param proxies: (可选) 自定义代理 [proxies={"https":"localhost:10809"}],支持 http 和 socks5（具体参考requests库的用法）
        :param port: (可选) 开启 http server 端口，用于网页端扫码登录. 提供此值时，将不再弹出或打印二维码
        :param email: (可选) 邮箱配置，参考 EMailConfig
        :param request_failed_delay: (可选) 由于网络异常导致的 request 异常，等待多少秒后重试
        :param requests_timeout: (可选) 应网友提议，添加 requests timeout 参数
        :param login_timeout: (可选) 登录超时时间，单位：秒。
        :param re_login: refresh_token 失效后是否继续登录（弹出二维码或邮件，需等待） fix #73

        level, use_aria2, proxies, port, email 可以通过 配置文件 配置默认值，在 <用户家目录>/.aligo/config.json5 中
        ```json5
        {
          "level": 10,
          "use_aria2": false,
          "proxies": {
            "https": "http://localhost:10809",
            // "https": "socks5://localhost:10808", # 不支持注释，写的时候删掉
          },
          "port": 8080,
          "email": ["邮箱地址", "防伪字符串"]
        }
        ```
        """
        config = aligo_config_folder / 'config.json5'
        if config.exists():
            config = json.loads(config.read_text(encoding='utf8'))
            if level == logging.DEBUG and 'level' in config:
                level = config.get('level')
            if not use_aria2:
                use_aria2 = config.get('use_aria2')
            if proxies is None:
                proxies = config.get('proxies')
            if port is None:
                port = config.get('port')
            if email is None:
                email = config.get('email')
            if request_failed_delay == 3:
                requests_timeout = config.get('request_failed_delay', 3)
            if requests_timeout is None:
                requests_timeout = config.get('requests_timeout')
            if requests_timeout is None:
                login_timeout = config.get('login_timeout')
            if re_login is True:
                re_login = config.get('re_login')

        super().__init__(
            name,
            refresh_token,
            show,
            level,
            use_aria2,
            proxies,
            port,
            email,
            request_failed_delay,
            requests_timeout,
            login_timeout,
            re_login,
        )



"""..."""
# 主机
API_HOST = 'https://api.aliyundrive.com'
AUTH_HOST = 'https://auth.aliyundrive.com'
WEBSV_HOST = 'https://websv.aliyundrive.com'
PASSPORT_HOST = 'https://passport.aliyundrive.com'
MEMBER_HOST = 'https://member.aliyundrive.com'

# 登录相关
NEWLOGIN_LOGIN_DO = '/newlogin/login.do'
NEWLOGIN_SMS_SEND_DO = '/newlogin/sms/send.do'
NEWLOGIN_SMS_LOGIN_DO = '/newlogin/sms/login.do'
NEWLOGIN_QRCODE_QUERY_DO = '/newlogin/qrcode/query.do'
NEWLOGIN_QRCODE_GENERATE_DO = '/newlogin/qrcode/generate.do'
V2_OAUTH_TOKEN_LOGIN = '/v2/oauth/token_login'
V2_OAUTH_AUTHORIZE = '/v2/oauth/authorize'
V2_ACCOUNT_TOKEN = '/v2/account/token'
TOKEN_REFRESH = '/token/refresh'
TOKEN_GET = '/token/get'
USERS_V1_USERS_DEVICE_RENEW_SESSION = '/users/v1/users/device/renew_session'
USERS_V1_USERS_DEVICE_LOGOUT = '/users/v1/users/device_logout'
USERS_V1_USERS_DEVICE_CREATE_SESSION = '/users/v1/users/device/create_session'
USERS_V2_USERS_DEVICE_LIST = '/users/v2/users/device_list'

# 基本信息
V2_USER_GET = '/v2/user/get'
ADRIVE_V1_USER_CONFIG_GET = '/adrive/v1/user_config/get'
V2_DRIVE_GET = '/v2/drive/get'
V2_DRIVE_GET_DEFAULT_DRIVE = '/v2/drive/get_default_drive'
V2_DRIVE_LIST_MY_DRIVES = '/v2/drive/list_my_drives'
V2_DATABOX_GET_PERSONAL_INFO = '/v2/databox/get_personal_info'
V2_DATABOX_GET_AUDIO_PLAY_INFO = '/v2/databox/get_audio_play_info'
V2_DATABOX_GET_VIDEO_PLAY_INFO = '/v2/databox/get_video_play_info'
BUSINESS_V1_USERS_VIP_INFO = '/business/v1.0/users/vip/info'
ADRIVE_V1_USER_DRIVECAPACITY_DETAILS = '/adrive/v1/user/driveCapacityDetails'

#  文件操作
V2_FILE_GET = '/v2/file/get'
V2_FILE_LIST = '/v2/file/list'
ADRIVE_V3_FILE_LIST = '/adrive/v3/file/list'
V3_FILE_UPDATE = '/v3/file/update'
V2_FILE_MOVE = '/v2/file/move'
V2_FILE_COPY = '/v2/file/copy'
V2_FILE_SEARCH = '/v2/file/search'
V2_FILE_CREATE = '/v2/file/create'
V2_FILE_COMPLETE = '/v2/file/complete'
ADRIVE_V2_FILE_CREATE = '/adrive/v2/file/create'
V2_FILE_CREATE_WITH_PROOF = '/v2/file/create_with_proof'
ADRIVE_V2_FILE_CREATEWITHFOLDERS = '/adrive/v2/file/createWithFolders'
V2_FILE_GET_UPLOAD_URL = '/v2/file/get_upload_url'
ADRIVE_V1_FILE_DUPLICATE_LIST = '/adrive/v1/file/duplicateList'
ADRIVE_V1_FILE_LISTTOCLEAN = '/adrive/v1/file/listToClean'
V2_FILE_GET_OFFICE_PREVIEW_URL = '/v2/file/get_office_preview_url'
ADRIVE_V1_FILE_GET_FOLDER_SIZE_INFO = '/adrive/v1/file/get_folder_size_info'

# 相册
ADRIVE_V1_USER_ALBUMS_INFO = '/adrive/v1/user/albums_info'
ADRIVE_V1_ALBUMHOME_ALBUMLIST = '/adrive/v1/albumHome/albumList'
ADRIVE_V1_ALBUM_DELETE = '/adrive/v1/album/delete'
ADRIVE_V1_ALBUM_GET = '/adrive/v1/album/get'
ADRIVE_V1_ALBUM_CREATE = '/adrive/v1/album/create'
ADRIVE_V1_ALBUM_LIST_FILES = '/adrive/v1/album/list_files'
ADRIVE_V1_ALBUM_UPDATE = '/adrive/v1/album/update'
ADRIVE_V1_ALBUM_ADD_FILES = '/adrive/v1/album/add_files'

V2_AIMS_SEARCH = '/v2/aims/search'

V2_FILE_GET_DOWNLOAD_URL = '/v2/file/get_download_url'

# 回收站
V2_RECYCLEBIN_TRASH = '/v2/recyclebin/trash'
V2_RECYCLEBIN_LIST = '/v2/recyclebin/list'
V2_RECYCLEBIN_RESTORE = '/v2/recyclebin/restore'

# 分享和收藏
V2_FILE_LIST_BY_CUSTOM_INDEX_KEY = '/v2/file/list_by_custom_index_key'
V2_SHARE_LINK_UPDATE = '/v2/share_link/update'
ADRIVE_V3_SHARE_LINK_LIST = '/adrive/v3/share_link/list'
ADRIVE_V2_SHARE_LINK_CREATE = '/adrive/v2/share_link/create'
ADRIVE_V2_SHARE_LINK_CANCEL = '/adrive/v2/share_link/cancel'
ADRIVE_V2_SHARE_LINK_GET_SHARE_BY_ANONYMOUS = '/adrive/v2/share_link/get_share_by_anonymous'
V2_SHARE_LINK_GET_SHARE_TOKEN = '/v2/share_link/get_share_token'
V2_FILE_GET_SHARE_LINK_DOWNLOAD_URL = '/v2/file/get_share_link_download_url'
ADRIVE_V2_SHARE_LINK_EXTRACT_CODE = '/adrive/v2/share_link/extract_code'
RECOMMEND_V1_SHARELINK_SEARCH = '/recommend/v1/shareLink/search'
ADRIVE_V2_FILE_LIST_BY_SHARE = '/adrive/v2/file/list_by_share'
ADRIVE_V2_FILE_GET_BY_SHARE = '/adrive/v2/file/get_by_share'

# 批量操作
V3_BATCH = '/v3/batch'
ADRIVE_V2_BATCH = '/adrive/v2/batch'

# 福利码
V1_USERS_REWARDS = '/v1/users/rewards'

# 获取路径
ADRIVE_V1_FILE_GET_PATH = '/adrive/v1/file/get_path'

# 在线解压缩相关
V2_ARCHIVE_UNCOMPRESS = '/v2/archive/uncompress'
V2_ARCHIVE_STATUS = '/v2/archive/status'

# m3u8
V2_FILE_GET_VIDEO_PREVIEW_PLAY_INFO = '/v2/file/get_video_preview_play_info'

# 模板
V2_TEMPLATE_TEST = '/v2/template/test'

# 参数
CLIENT_ID = '25dzX3vbYqktVxyX'
UNI_PARAMS = {'appName': 'aliyun_drive'}
UNI_HEADERS = {
    'Referer': 'https://aliyundrive.com',
    'User-Agent': ('Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) '
                   'Chrome/91.0.4472.114 Safari/537.36'),
    # 没有此请求头，list file 获取不到 download_url 字段，url 不支持断点续传
    'x-canary': 'client=web,app=adrive,version=v4.1.0',
}



"""..."""
import json
import logging
import subprocess
import traceback
from dataclasses import asdict, is_dataclass
from typing import Generic, List, Iterator, Dict, Callable, Union, Type

import requests

from aligo.core import *
from aligo.core.Config import *
from aligo.request import *
from aligo.response import *
from aligo.types import *
from aligo.types.DataClass import DataType
from aligo.types.Enum import *


class BaseAligo:
    """..."""

    def __init__(
            self,
            name: str = 'aligo',
            refresh_token: str = None,
            show: Callable[[str], None] = None,
            level: int = logging.DEBUG,
            use_aria2: bool = False,
            proxies: Dict = None,
            port: int = None,
            email: EMailConfig = None,
            request_failed_delay: float = 3,
            requests_timeout: float = None,
            login_timeout: float = None,
            re_login: bool = True
    ):
        """
        BaseAligo
        :param name: (可选, 默认: aligo) 配置文件名称, 便于使用不同配置文件进行身份验证
        :param refresh_token:
        :param show: (可选) 显示二维码的函数
        :param level: (可选) 控制控制台输出
        :param use_aria2: [bool] 是否使用 aria2 下载
        :param proxies: (可选) 自定义代理 [proxies={"https":"localhost:10809"}],支持 http 和 socks5（具体参考requests库的用法）
        :param port: (可选) 开启 http server 端口，用于网页端扫码登录. 提供此值时，将不再弹出或打印二维码
        :param email: (可选) 邮箱配置，参考 EMailConfig
        :param request_failed_delay: 请求失败后，延迟时间，单位：秒
        :param requests_timeout: same as requests timeout
        :param login_timeout: 登录超时时间，单位：秒
        :param re_login: refresh_token 失效后是否继续登录（弹出二维码或邮件，需等待） fix #73
        """
        self._auth: Auth = Auth(  # type: ignore
            name=name,
            refresh_token=refresh_token,
            show=show,
            level=level,
            proxies=proxies,
            port=port,
            email=email,
            request_failed_delay=request_failed_delay,
            requests_timeout=requests_timeout,
            login_timeout=login_timeout,
            re_login=re_login,
        )
        # 因为 self._auth.session 没有被重新赋值, 所以可以这么用
        self._session: requests.Session = self._auth.session
        # 在刷新 token 时, self._auth.token 被重新赋值, 而 self._token 却不会被更新
        # self._token: Token = self._auth.token
        self._user: Optional[BaseUser] = None
        self._personal_info: Optional[GetPersonalInfoResponse] = None
        self._default_drive: Optional[BaseDrive] = None

        if use_aria2:
            try:
                subprocess.run(['aria2c', '-h'], capture_output=True)
                self._has_aria2c = True
                self._auth.log.info('发现 aria2c, 将使用 aria2c 下载文件')
            except FileNotFoundError:
                self._auth.log.warning('未发现 aria2c')
                self._has_aria2c = False
        else:
            self._has_aria2c = False

    def _post(
            self,
            path: str,
            host: str = API_HOST,
            body: Union[DataType, Dict] = None,
            headers: dict = None,
            ignore_auth: bool = False,
            params: dict = None,
    ) -> requests.Response:
        """统一处理数据类型和 drive_id"""
        if body is None:
            body = {}
        elif isinstance(body, DataClass):
            body = asdict(body)

        if 'drive_id' in body and body['drive_id'] is None:
            # 如果存在 attr drive_id 并且它是 None，并将 default_drive_id 设置为它
            body['drive_id'] = self.default_drive_id

        return self._auth.post(path=path, host=host, body=body, headers=headers, ignore_auth=ignore_auth, params=params)

    @property
    def default_drive_id(self):
        """默认 drive_id"""
        return self._auth.token.default_drive_id

    @property
    def default_sbox_drive_id(self):
        """默认保险箱 drive_id"""
        return self._auth.token.default_sbox_drive_id

    @property
    def user_name(self):
        """用户名"""
        return self._auth.token.user_name

    @property
    def user_id(self):
        """用户 id"""
        return self._auth.token.user_id

    @property
    def nick_name(self):
        """昵称"""
        return self._auth.token.nick_name

    def _result(self, response: requests.Response,
                cls: Generic[DataType],
                status_code: Union[List, int] = 200, field: str = None) -> Union[Null, DataType]:
        """统一处理响应

        :param response:
        :param cls:
        :param status_code:
        :return:
        """
        if isinstance(status_code, int):
            status_code = [status_code]
        if response.status_code in status_code:
            text = response.text
            if not text.startswith('{'):
                return cls()
            try:
                # noinspection PyProtectedMember
                d = json.loads(text)
                if field:
                    for i in field.split('.'):
                        d = d[i]
                if isinstance(d, list):
                    return [DataClass.fill_attrs(cls, i) for i in d]
                return DataClass.fill_attrs(cls, d)
            except TypeError:
                self._auth.debug_log(response)
                self._auth.log.error(cls)
                traceback.print_exc()
        self._auth.log.warning(f'{response.status_code} {response.text[:200]}')
        return Null(response)

    def _list_file(
            self, path: str, body: Union[DataClass, Dict],
            resp_type: Generic[DataType], headers: dict = None, params: dict = None) -> Iterator[DataType]:
        """
        枚举文件: 用于统一处理 1.文件列表 2.搜索文件列表 3.收藏列表 4.回收站列表
        :param path: [str] 批量处理的路径
        :param body: [DataClass] 批量处理的参数
        :param resp_type: [Callable] 响应类型
        :return: [Iterator[DataType]] 响应结果

        如何判断请求失败与否（适用于所有使用此方法的上层方法）：
        >>> from aligo import Aligo
        >>> ali = Aligo()
        >>> result = ali.get_file_list('<file_id>')
        >>> if isinstance(result[-1], Null):
        >>>     print('请求失败')
        """
        response = self._post(path, body=body, headers=headers, params=params)
        file_list = self._result(response, resp_type)
        if isinstance(file_list, Null):
            yield file_list
            return
        yield from file_list.items
        if file_list.next_marker != '':
            if isinstance(body, dict):
                body['marker'] = file_list.next_marker
            else:
                body.marker = file_list.next_marker
            yield from self._list_file(path=path, body=body, resp_type=resp_type, headers=headers, params=params)

    def _core_get_file(self, body: GetFileRequest) -> BaseFile:
        """获取文件信息, 其他类中可能会用到, 所以放到基类中"""
        response = self._post(V2_FILE_GET, body=body)
        return self._result(response, BaseFile)

    def get_personal_info(self) -> GetPersonalInfoResponse:
        """
        获取个人信息
        :return: [GetPersonalInfoResponse]
        """
        response = self._post(V2_DATABOX_GET_PERSONAL_INFO)
        return self._result(response, GetPersonalInfoResponse)

    _BATCH_COUNT = 100

    @staticmethod
    def _list_split(ll: List[DataType], n: int) -> List[List[DataType]]:
        rt = []
        for i in range(0, len(ll), n):
            rt.append(ll[i:i + n])
        return rt

    def batch_request(self, body: BatchRequest, body_type: Type[DataType]) -> Iterator[BatchSubResponse[DataType]]:
        """
        批量请求：官方最大支持 100 个请求，所以这里按照 100 个一组进行分组，然后分别请求，使用时无需关注这个。
        :param body:[BatchRequest] 批量请求的参数
        :param body_type: [DataType] 批量请求的参数类型
        :return: [Iterator[DataType]]

        如何判断请求失败与否（适用于所有使用此方法的上层方法）：
        >>> from aligo import Aligo
        >>> ali = Aligo()
        >>> result = ali.batch_get_files(['<file1_id>', '<file2_id>'])
        >>> if isinstance(result[-1], Null):
        >>>     print('请求失败')
        """
        for request_list in self._list_split(body.requests, self._BATCH_COUNT):
            response = self._post(V3_BATCH, body={
                "requests": [
                    {
                        "body": asdict(request.body) if is_dataclass(request.body) else request.body,
                        "headers": request.headers,
                        "id": request.id,
                        "method": request.method,
                        "url": request.url
                    } for request in request_list
                ],
                "resource": body.resource
            })

            if response.status_code != 200:
                yield Null(response)
                return

            for batch in response.json()['responses']:
                i = DataClass.fill_attrs(BatchSubResponse, batch)
                if i.body:
                    try:
                        # 不是都会成功
                        # eg: {
                        #       'code': 'AlreadyExist.File',
                        #       'message': "The resource file has already exists. drive has the same file,
                        #                   can't update, file_id 609887cca951bf4feca54c6ebd0a91a03b826949"
                        # }
                        # status 409
                        i.body = DataClass.fill_attrs(body_type, i.body)
                    except TypeError:
                        # self._auth.log.warning(i)
                        pass
                yield i



"""文件相关"""
import os
from typing import List, overload, Union, Callable

from aligo.core import *
from aligo.core.Config import *
from aligo.request import *
from aligo.response import *
from aligo.types import *
from aligo.types.Enum import *


class File(Core):
    """..."""

    @overload
    def get_file(self, file_id: str, drive_id: str = None) -> BaseFile:
        """
        获取文件
        :param file_id: [str] 文件ID
        :param drive_id: Optional[str] 存储桶ID
        :return: [BaseFile] 文件对象

        用法示例:
        >>> from aligo import Aligo, BaseFile
        >>> ali = Aligo()
        >>> file = ali.get_file('file_id')
        >>> assert isinstance(file, BaseFile)
        >>> print(file)
        """

    @overload
    def get_file(self, body: GetFileRequest) -> BaseFile:
        """
        获取文件
        :param body: [GetFileRequest] 获取文件请求体
        :return: [BaseFile] 文件对象

        用法示例:
        >>> from aligo import Aligo, BaseFile, GetFileRequest
        >>> ali = Aligo()
        >>> file = ali.get_file(body=GetFileRequest(file_id='file_id'))
        >>> assert isinstance(file, BaseFile)
        >>> print(file)
        """

    def get_file(self,
                 file_id: str = None,
                 drive_id: str = None,
                 body: GetFileRequest = None,
                 **kwargs) -> BaseFile:
        """get_file"""
        if body is None:
            body = GetFileRequest(
                file_id=file_id, drive_id=drive_id,
                **kwargs
            )
        return self._core_get_file(body)

    @overload
    def get_file_list(self, parent_file_id: str = 'root', drive_id: str = None, **kwargs) -> List[BaseFile]:
        """
        获取文件列表
        :param parent_file_id: Optional[str] 文件夹ID，默认为根目录
        :param drive_id: Optional[str] 存储桶ID
        :param kwargs: [dict] 其他参数
        :return: [List[BaseFile]] 文件列表

        用法示例:
        >>> from aligo import Aligo
        >>> ali = Aligo()
        >>> file_list = ali.get_file_list('<parent_file_id>')
        >>> assert isinstance(file_list, list)
        >>> print(file_list)
        """

    @overload
    def get_file_list(self, body: GetFileListRequest) -> List[BaseFile]:
        """
        获取文件列表
        :param body: [GetFileListRequest] 获取文件列表请求体
        :return: [List[BaseFile]] 文件列表

        用法示例:
        >>> from aligo import Aligo, BaseFile, GetFileListRequest
        >>> ali = Aligo()
        >>> file_list = ali.get_file_list(body=GetFileListRequest(parent_file_id='<parent_file_id>'))
        >>> assert isinstance(file_list, list)
        >>> print(file_list)
        """

    def get_file_list(self, parent_file_id: str = 'root', drive_id: str = None, body: GetFileListRequest = None,
                      **kwargs) -> List[BaseFile]:
        """get_file_list"""
        if body is None:
            body = GetFileListRequest(drive_id=drive_id, parent_file_id=parent_file_id, **kwargs)
        result = self._core_get_file_list(body)
        return list(result)

    def batch_get_files(self, file_id_list: List[str], drive_id: str = None) -> List[BatchSubResponse]:
        """
        批量获取文件
        :param file_id_list: [List[str]] 文件ID列表
        :param drive_id: [str] 存储桶ID
        :return: [List[BatchSubResponse]] 批量获取文件响应列表

        用法示例:
        >>> from aligo import Aligo, Null
        >>> ali = Aligo()
        >>> file_list = ali.batch_get_files(['file_id1', 'file_id2'])
        >>> assert isinstance(file_list, list)
        >>> # 实际使用中一般不需要判断，这里写出来只是提供一个判断示例
        >>> if isinstance(file_list[-1], Null):
        >>>     print('出现错误')
        >>> for e in file_list:
        >>>     assert isinstance(e, BatchSubResponse)
        >>> print(file_list)
        """
        body = BatchGetFileRequest(file_id_list=file_id_list, drive_id=drive_id)
        result = self._core_batch_get_files(body)
        return list(result)

    def get_folder_by_path(
            self, path: str = '/', parent_file_id: str = 'root', create_folder: bool = False,
            check_name_mode: CheckNameMode = 'refuse', drive_id: str = None
    ) -> Union[BaseFile, CreateFileResponse, None]:
        """
        根据文件路径，获取网盘文件对象
        :param path: [str] 完整路径，无需以 '/' 开始或结束
        :param parent_file_id: Optional[str] 父文件夹ID，默认为根目录，意思是基于根目录查找
        :param create_folder:  Optional[bool] 不存在是否创建，默认：True. 此行为效率最高
        :param check_name_mode: Optional[CheckNameMode] 检查名称模式，默认为 'refuse'
        :param drive_id: Optional[str] 存储桶ID，一般情况下，drive_id 参数都无需提供
        :return: 文件对象，或创建文件夹返回的对象，或 None

        用法示例:
        >>> from aligo import Aligo
        >>> ali = Aligo()
        >>> folder = ali.get_folder_by_path('path/to/folder')
        >>> assert isinstance(folder, BaseFile)
        >>> print(folder)
        """
        path = path.strip('/')
        if not path:
            return self.get_file(file_id=parent_file_id, drive_id=drive_id)
        folder = None
        if create_folder:
            for folder_name in path.split('/'):
                folder = self._core_create_folder(
                    CreateFolderRequest(
                        name=folder_name, parent_file_id=parent_file_id,
                        check_name_mode=check_name_mode, drive_id=drive_id
                    )
                )
                parent_file_id = folder.file_id
            return folder
        else:
            for folder_name in path.split('/'):
                folders = self.get_file_list(parent_file_id=parent_file_id, drive_id=drive_id, type='folder')
                # 获取目标 folder
                for folder in folders:
                    if folder.name == folder_name:
                        parent_file_id = folder.file_id
                        break
                else:
                    return None
            return folder

    def get_file_by_path(self, path: str = '/', parent_file_id: str = 'root',
                         check_name_mode: CheckNameMode = 'refuse',
                         drive_id: str = None) -> Optional[BaseFile]:
        """
        根据路径获取云盘文件对象, 先找到啥就返回啥（早期可能存在同名文件（夹）），否则返回None
        :param path: [str] 路径，无需以'/'开头
        :param parent_file_id: Optional[str] 父文件夹ID，默认为根目录，意思是基于根目录查找
        :param check_name_mode: Optional[CheckNameMode] 检查名称模式，默认为 'refuse'
        :param drive_id: Optional[str] 存储桶ID
        :return: [BaseFile] 文件对象，或 None

        用法示例:
        >>> from aligo import Aligo
        >>> ali = Aligo()
        >>> file = ali.get_file_by_path('path/to/file')
        >>> assert isinstance(file, BaseFile)
        >>> print(file)
        """
        path = path.strip('/')
        folder_path, file_name = os.path.split(path)
        folder = None
        if folder_path != '':
            folder = self.get_folder_by_path(
                folder_path, parent_file_id=parent_file_id, create_folder=False,
                check_name_mode=check_name_mode, drive_id=drive_id
            )
            if folder is None:
                return None
            parent_file_id = folder.file_id

        if file_name == '':
            return folder

        file_list = self.get_file_list(parent_file_id=parent_file_id, drive_id=drive_id, type='file')

        for file in file_list:
            if file_name == file.name:
                return file

        return None

    def walk_files(self, callback: Callable[[str, BaseFile], None], parent_file_id: str = 'root', drive_id: str = None,
                   _path: str = ''):
        """遍历指定文件夹下所有文件

        :param drive_id:
        :param parent_file_id: 文件夹 id，默认 root
        :param callback: 回调函数
        :param _path: 当前文件基于指定文件夹的路径
        :return:
        """
        for f in self._core_get_file_list(GetFileListRequest(parent_file_id=parent_file_id, drive_id=drive_id)):
            if f.type == 'file':
                callback(_path, f)
                continue
            self.walk_files(callback, parent_file_id=f.file_id, drive_id=drive_id, _path=os.path.join(_path, f.name))

    def get_folder_size_info(self, file_id: str, drive_id: str = None) -> FolderSizeInfo:
        """获取文件夹信息，文件夹个数，以及文件个数及其大小，不递归"""
        response = self._post(ADRIVE_V1_FILE_GET_FOLDER_SIZE_INFO, body={'file_id': file_id, 'drive_id': drive_id})
        return self._result(response, FolderSizeInfo)



"""..."""
import os
from typing import List, overload, Callable

from aligo.core import *
from aligo.error import AligoException
from aligo.request import *
from aligo.response import *
from aligo.types import *


class Download(Core):
    """..."""

    def get_download_url(self,
                         file_id: str,
                         file_name: str = None,
                         expire_sec: int = 14400,
                         drive_id: str = None,
                         ) -> GetDownloadUrlResponse:
        """
        获取下载链接: 一般在已知 file_id 或 download_url 失效时使用
        :param file_id: [str] 文件 id
        :param file_name: Optional[str] 文件名
        :param expire_sec: Optional[int] 下载链接有效时间, 默认为 4 小时, 这也是允许的最大值
        :param drive_id: Optional[str] 文件所在的网盘 id
        :return: [GetDownloadUrlResponse]

        用法示例:
        >>> from aligo import Aligo
        >>> ali = Aligo()
        >>> result = ali.get_download_url(file_id='<file_id>')
        >>> print(result)
        """
        body = GetDownloadUrlRequest(
            file_id=file_id,
            drive_id=drive_id,
            file_name=file_name,
            expire_sec=expire_sec,
        )
        return self._core_get_download_url(body)

    def batch_download_url(self,
                           file_id_list: List[str],
                           expire_sec: int = 14400,
                           drive_id=None) -> List[BatchDownloadUrlResponse]:
        """
        批量获取下载链接
        :param file_id_list: [List[str]] 文件 id 列表
        :param expire_sec: [int] 下载链接有效时间, 默认为 4 小时, 这也是允许的最大值
        :param drive_id: [str] 文件所在的网盘 id
        :return: [List[BatchDownloadUrlResponse]]

        用法示例:
        >>> from aligo import Aligo
        >>> ali = Aligo()
        >>> result = ali.batch_download_url(file_id_list=['<file1_id>', '<file2_id>'])
        >>> print(result)
        """
        body = BatchDownloadUrlRequest(
            drive_id=drive_id,
            file_id_list=file_id_list,
            expire_sec=expire_sec
        )
        result = self._core_batch_download_url(body)
        return list(result)

    def download_folder(self, folder_file_id: str, local_folder: str = '.', drive_id: str = None,
                        file_filter: Callable[[BaseFile], bool] = lambda x: False) -> str:
        """
        下载文件夹
        :param folder_file_id: [str] 文件夹 id
        :param local_folder: [str] 本地文件夹路径, 默认为当前目录, 即下载到哪里
        :param drive_id: [str] 文件夹所在的网盘 id
        :param file_filter: 文件过滤函数
        :return: [str] 本地文件夹路径

        用法示例:
        >>> from aligo import Aligo
        >>> ali = Aligo()
        >>> result = ali.download_folder(folder_file_id='<folder_file_id>')
        >>> print(result)
        """
        if folder_file_id != 'root':
            folder = self._core_get_file(GetFileRequest(file_id=folder_file_id, drive_id=drive_id))
            local_folder = os.path.join(local_folder, self._del_special_symbol(folder.name))
        return self.__download_folder(folder_file_id, local_folder, drive_id, file_filter=file_filter)

    def __download_folder(self, folder_file_id: str, local_folder: str = '.', drive_id: str = None,
                          file_filter: Callable[[BaseFile], bool] = lambda x: False) -> str:
        """下载文件夹"""
        # 创建文件夹, 即使文件夹为空
        os.makedirs(local_folder, exist_ok=True)
        files = []
        for file in self._core_get_file_list(GetFileListRequest(parent_file_id=folder_file_id, drive_id=drive_id)):
            if file_filter(file):
                continue
            if file.type == 'folder':
                self.__download_folder(folder_file_id=file.file_id,
                                       local_folder=os.path.join(local_folder, self._del_special_symbol(file.name)))
                continue
            files.append(file)
        self.download_files(files, local_folder=local_folder)
        return os.path.abspath(local_folder)

    @overload
    def download_file(self, *, file_path: str, url: str) -> str:
        """
        根据下载地址下载文件
        :param file_path: [str] 文件路径
        :param url: [str] 下载地址
        :return: [str] 本地文件路径

        用法示例:
        >>> from aligo import Aligo
        >>> ali = Aligo()
        >>> result = ali.download_file(file_path='<file_path>', url='<url>')
        >>> print(result)
        """

    @overload
    def download_file(self, *, file_id: str, local_folder: str = '.') -> str:
        """
        根据 file_id 下载文件
        :param file_id: [str] 文件 id
        :param local_folder: Optional[str] 本地文件夹路径, 默认为当前目录, 即下载到哪里
        :return: [str] 本地文件路径

        用法示例:
        >>> from aligo import Aligo
        >>> ali = Aligo()
        >>> result = ali.download_file(file_id='<file_id>')
        >>> print(result)
        """

    @overload
    def download_file(self, *, file: BaseFile, local_folder: str = '.') -> str:
        """
        根据 BaseFile 对象下载文件
        :param file: [BaseFile] 文件对象
        :param local_folder: Optional[str] 本地文件夹路径, 默认为当前目录, 即下载到哪里
        :return: [str] 本地文件路径

        用法示例:
        >>> from aligo import Aligo
        >>> ali = Aligo()
        >>> # ll = ali.get_file_list()
        >>> # result = ali.download_file(ll[-1])
        >>> result = ali.download_file(file=BaseFile(file_id='<file_id>'))
        >>> print(result)
        """

    def download_file(
            self, *, file_path: str = None, url: str = None,
            local_folder: str = '.', file_id: str = None,
            file: BaseFile = None, drive_id=None
    ) -> str:
        """download_file"""
        if file_id:
            file = self._core_get_file(GetFileRequest(file_id=file_id, drive_id=drive_id))

        if file:
            if file.type == 'folder':
                raise AligoException('文件类型不对：期待文件，得到的是文件夹')
            file_path = os.path.join(local_folder, file.name)
            url = file.download_url or file.url

        return self._core_download_file(file_path, url)



"""..."""
from dataclasses import dataclass

from .DataClass import DataClass


@dataclass
class FieldsInfo(DataClass):
    """..."""
    image_count: int = 0
    video_meta_processed: str = None
    story_ids: str = None



"""..."""
from dataclasses import dataclass

from aligo.types import DataClass
from aligo.types.Enum import AlbumFileListType, OrderDirection


@dataclass
class AlbumListFilesRequest(DataClass):
    """..."""
    album_id: str = None
    fields: str = "*"
    filter: str = ""
    image_thumbnail_process: str = "image/resize,w_400/format,jpeg"
    image_url_process: str = "image/resize,w_1920/format,jpeg"
    limit: int = 100
    order_by: AlbumFileListType = "joined_at"
    order_direction: OrderDirection = "DESC"
    video_thumbnail_process: str = "video/snapshot,t_0,f_jpg,ar_auto,w_1000"



"""..."""
from dataclasses import dataclass

from aligo.types import *
from aligo.types.Enum import *


@dataclass
class RenameFileRequest(DataClass):
    """..."""
    name: str
    file_id: str
    check_name_mode: CheckNameMode = 'refuse'
    drive_id: str = None



"""GetFilePathRequest"""
from dataclasses import dataclass

from aligo.types import DataClass


@dataclass
class GetFilePathRequest(DataClass):
    """GetFilePathRequest"""
    file_id: str = None
    drive_id: str = None



"""发送邮件模块"""
import smtplib
import ssl
import time
from email.mime.image import MIMEImage
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from email.utils import formataddr


def send_email(
        receiver: str, title: str, content: str, qr_data: bytes,
        email_user: str, email_password: str, email_host: str, email_port: int,
):
    """发送邮件"""
    msg_root = MIMEMultipart()
    msg_root['From'] = formataddr(('aligo notify', email_user))
    msg_root['To'] = formataddr((receiver, receiver))
    msg_root['Subject'] = f'[阿里云盘/{title}] 扫码登录'

    msg_root.attach(
        MIMEText(f'<div align="center"><h3>{content}</h3><img style="max-width: 100%" src="cid:qrcode"></div>', 'html'))

    msg_image = MIMEImage(qr_data, 'png')
    msg_image.add_header('Content-ID', '<qrcode>')

    msg_root.attach(msg_image)

    try:
        smtp = smtplib.SMTP_SSL(email_host, email_port)
    except ssl.SSLError:
        smtp = smtplib.SMTP(email_host, email_port)
    smtp.login(email_user, email_password)
    for i in range(1, 4):
        try:
            result = smtp.sendmail(
                email_user,
                [receiver],
                msg_root.as_bytes()
            )
            return result
        except smtplib.SMTPServerDisconnected:
            time.sleep(i * 3)
    raise f'邮件发送失败 {receiver}'
